-- Author      : Nogarder
-- Create Date : 2015/3/1 22:37:36
local pairs=pairs
local select=select
local next=next

local RS_EventHandler_MainFrame=CreateFrame("Frame",nil,UIParent)
RS_EventHandler_MainFrame:SetWidth(1)
RS_EventHandler_MainFrame:SetHeight(1)
RS_EventHandler_MainFrame:Show()


--main
RS.EventHandler={}
--func lists
local RS_EventHandler_EventFuncs={
	[0]={}, --system event
	[1]={}, --custom event
}
RS.EventHandler.UnregisterEventByName=function(eventtype,name)
	local funclist=RS_EventHandler_EventFuncs[eventtype]
	if(funclist)then
		for event,list in pairs(funclist) do
			list[name]=nil
		end
	end
end
----register function
RS.EventHandler.RegisterEvent=function(eventtype,event,name,func)
	local funclist=RS_EventHandler_EventFuncs[eventtype]
	--register
	if(func)then
		if(not funclist)then
			RS_EventHandler_EventFuncs[eventtype]={}
			funclist=RS_EventHandler_EventFuncs[eventtype]
		end
		if(not funclist[event])then
			funclist[event]={}
		end
		funclist[event][name]=func
		if(eventtype==0)then --is system event
			--register system event
			RS_EventHandler_MainFrame:RegisterEvent(event)
		end
	else
		--unregister if func is nil
		if(funclist[event])then
			funclist[event][name]=nil
			if(not next(funclist[event]))then
				funclist[event]=nil
				if(not next(RS_EventHandler_EventFuncs[eventtype]))then
					RS_EventHandler_EventFuncs[eventtype]=nil
				end
				if(eventtype==0)then --system event
					--no longer listening, unregister
					RS_EventHandler_MainFrame:UnregisterEvent(event)
				end
			end
		end
	end
end
--
local RealFireEvent=function(eventtype,event,...)
	local eventlist=RS_EventHandler_EventFuncs[eventtype]
	local funclist=eventlist[event]
	if(event~="Any")then
		if(funclist)then
			for _,func in pairs(funclist) do
				func(...)
			end
		end
		--function registered to "any" would be fired in any other event
		--with event as parameter 
		local funclist=eventlist["Any"]
		if(funclist)then
			for _,func in pairs(funclist) do
				func(event,...)
			end
		end
	else
		--fire with "Any" as parameter
		for _,func in pairs(funclist) do
			func(event,...)
		end
	end
end
--
RS.EventHandler.FireEvent=function(eventtype,event,...)
	--"any" event would fire every event in this eventtype
	if(event=="Any")then
		for eventname,funclist in pairs(RS_EventHandler_EventFuncs[eventtype])do
			RealFireEvent(eventtype,eventname,...)
		end
	else
		RealFireEvent(eventtype,event,...)
	end
end
--
local RS_EventHandler_FireEvent=RS.EventHandler.FireEvent
--for event handler
RS_EventHandler_MainFrame:HookScript("OnEvent",function(_,event,...)
	RS_EventHandler_FireEvent(0,event,...)
end)

RS_EventHandler_MainFrame:HookScript("OnUpdate",function(...)
	RS_EventHandler_FireEvent(1,"RS_MainFrame_OnUpdate",...)
end)
