-- Author      : Nogarder
-- Create Date : 2015/3/9 20:25:56
RS.RegisterPageFunc("FontEditor",function()

	local pairs=pairs
	local CopyTable=CopyTable
	--
	local Localization=RS.Localization
	local Tooltips=Localization.Tooltips
	--
	local parent=RS.OptionsMain
	local main=RS.Template.CreateFrameWithBar(parent,Localization["FontEditor"],210,300,nil,nil,nil)
	main:Hide()
	local title=main.Scroll.Title
	local page=main.Frame
	--
	--elements
	local step=-20
	--save
	local savebutton=RS.Template.CreateSpecialButton(page,65,15,"Save")
	savebutton:SetPoint("TOPLEFT",page,"TOPLEFT",5,-5)
	--reset
	local resetbutton=RS.Template.CreateSpecialButton(page,65,15,"Reset")
	resetbutton:SetPoint("TOPRIGHT",page,"TOPRIGHT",-5,-5)
	--cancel
	local cancelbutton=RS.Template.CreateSpecialButton(page,65,15,"Cancel")
	cancelbutton:SetPoint("LEFT",savebutton,"RIGHT",5,0)
	cancelbutton:SetPoint("RIGHT",resetbutton,"LEFT",-5,0)
	--font
	local GetFont=function()
		local RS_FontLib={
			["System:Unit"] = UNIT_NAME_FONT,
			["System:Damage"] = DAMAGE_TEXT_FONT,
			["System:Chat"] = ChatFontNormal:GetFont(),
			["System:Default"] = NAMEPLATE_FONT,
		}
		--SharedMedia-3.0 support
		if(LibStub)then
			local sharemedia=LibStub:GetLibrary("LibSharedMedia-3.0",1)
			if(sharemedia)then
				for name,value in pairs(sharemedia.MediaTable.font) do
					RS_FontLib[name]=value
				end
			end
		end
		--convert into new format
		local fontlib={}
		local i=0
		for name,value in pairs(RS_FontLib)do
			i=i+1
			fontlib[i]={
				Text=name,
				Value=value,
				Font=value,
			}
		end
		return fontlib
	end
	local fontselector=RS.Template.CreateDropDownEditor(page,200,Localization["NameOrPath"],Tooltips["MustExistWhenLoad"],GetFont)
	fontselector:SetPoint("TOPLEFT",savebutton,"BOTTOMLEFT",0,step)
	fontselector.FreeEdit=true
	--font mode
	local GetFontMode=function()
		local RS_FontMode={
			["None"]="",
			["Outline"]="OUTLINE",
			["Thick"]="THICKOUTLINE",
			--["Mono"]="MONOCHROME",  --this would crash client for unknow reason
			["Outline+Thick"]="OUTLINE,THICKOUTLINE",
			["Outline+Mono"]="OUTLINE,MONOCHROME",
			["Thick+Mono"]="THICKOUTLINE,MONOCHROME",
			["Outline+Thick+Mono"]="OUTLINE,THICKOUTLINE,MONOCHROME",
		}
		--convert into new format
		local fontmode={}
		local i=0
		for name,value in pairs(RS_FontMode)do
			i=i+1
			fontmode[i]={
				Text=name,
				Value=value
			}
		end
		return fontmode
	end
	local modeselector=RS.Template.CreateDropDownEditor(page,200,Localization["RenderMode"],Tooltips["RenderMode"],GetFontMode)
	modeselector:SetPoint("TOPLEFT",fontselector,"BOTTOMLEFT",0,step)
	--shadowxslider
	local shadowxslider=RS.Template.CreateSliderEdit(page,150,50,20,-10,10,1,0,Localization["ShadowOffsetX"],Tooltips["ShadowOffsetX"])
	shadowxslider:SetPoint("TOPLEFT",modeselector,"BOTTOMLEFT",0,step)
	--shadowyslider
	local shadowyslider=RS.Template.CreateSliderEdit(page,150,50,20,-10,10,1,0,Localization["ShadowOffsetY"],Tooltips["ShadowOffsetY"])
	shadowyslider:SetPoint("TOPLEFT",shadowxslider,"BOTTOMLEFT",0,step)
	--preview
	local previewbg=page:CreateTexture(nil,"BACKGROUND")
	previewbg:SetTexture(0,0,0,1)
	previewbg:SetPoint("TOPLEFT",shadowyslider,"BOTTOMLEFT",0,step)
	previewbg:SetPoint("BOTTOMRIGHT",page,"BOTTOMRIGHT",-5,5)
	local preview1=RS.Template.CreateText(page,Localization["PreviewText"])
	preview1:SetPoint("TOPLEFT",shadowyslider,"BOTTOMLEFT",5,step-5)
	local preview2=RS.Template.CreateText(page,Localization["PreviewText"])
	preview2:SetPoint("TOPLEFT",preview1,"BOTTOMLEFT",0,-5)
	--
	--function
	local RefreshPreview=function()
		local fontdata={
			Font=fontselector.Value,
			--mode
			Mode=modeselector.Value,
			--shadowxslider
			ShadowX=shadowxslider.Value,
			ShadowY=shadowyslider.Value,
		}

		preview1:SetFont(fontdata.Font,12,fontdata.Mode)
		preview1:SetShadowOffset(fontdata.ShadowX,fontdata.ShadowY)
		preview2:SetFont(fontdata.Font,12,fontdata.Mode)
		preview2:SetShadowOffset(fontdata.ShadowX,fontdata.ShadowY)
	end
	--
	local LoadData=function(default)
		--page
		if(default)then
			entry={
				Font=RS.TextFont,
				--mode
				Mode=RS.TextMode,
				--shadowxslider
				ShadowX=1,
				ShadowY=1,
			}
		else
			entry=RS_MiscSetting.FontSetting
		end
		--font
		local value=entry.Font
		RS.Template.SetDropDownEditor(fontselector,value)
		--mode
		local value=entry.Mode
		RS.Template.SetDropDownEditor(modeselector,value)
		--shadowxslider
		local value=entry.ShadowX
		RS.Template.SetSliderEdit(shadowxslider,value)
		local value=entry.ShadowY
		RS.Template.SetSliderEdit(shadowyslider,value)
		--color
		--
		RefreshPreview()
	end
	local SaveData=function()
		local entry=RS_MiscSetting.FontSetting
		--font
		entry.Font=fontselector.Value
		--mode
		entry.Mode=modeselector.Value
		--shadowxslider
		entry.ShadowX=shadowxslider.Value
		entry.ShadowY=shadowyslider.Value
		--color
		--fire event
		RS.EventHandler.FireEvent(1,"RS_SettingUpdated")
	end
	--
	savebutton:HookScript("OnMouseUp",function()
		RS.Animation.ShowFrame(main)
		SaveData()
	end)
	cancelbutton:HookScript("OnMouseUp",function()
		RS.Animation.ShowFrame(main)
		LoadData()
	end)
	resetbutton:HookScript("OnMouseUp",function()
		RS.Animation.ShowFrame(main)
		LoadData(true)
	end)
	--
	fontselector.OnValueUpdateFunc=RefreshPreview
	modeselector.OnValueUpdateFunc=RefreshPreview
	shadowxslider.OnValueUpdateFunc=RefreshPreview
	shadowyslider.OnValueUpdateFunc=RefreshPreview
	--
	main.OnSetFunc=function()
		LoadData()
	end
	--
	RS.RegisterPage("FontEditor",main)

end)