-- Author      : Nogarder
-- Create Date : 2015/3/2 1:48:16

--playerdata={
--	Name=name,
--	RoleIcon=role texture,
--	SpecIcon=spec texture,
--	SpellList={
--		[SpellID]=CoolDown
--		}
--	}
--SelectedSpell={
--		[SpellID]=CoolDown,
--}
--}

local RS_Template_SetPlayerFrame=function(button,playerdata,selected)
	button.PlayerData=playerdata

	RS.SpecAndSpells.SetPortrait(button.PortraitTexture,playerdata.Name)

	button.Label:SetText(playerdata.Name)

	RS.SetTextColor(button.Label,playerdata.Color)

	RS.SetTexCoord(button.RoleTexture,playerdata.RoleIcon)

	button.SpecTexture:SetTexture(playerdata.SpecIcon)

	if(selected)then
		RS.Template.ShowLinerBorder(button)
	else
		RS.Template.HideLinerBorder(button)
	end
end

local RS_Template_ClearPlayerFrame=function(button)
	button.PlayerData=nil

	button.PortraitTexture:SetTexture(nil)
	button.Label:SetText("+")

	button.RoleTexture:SetTexCoord(0,0,0,0)

	button.SpecTexture:SetTexture(nil)

	RS.Template.HideLinerBorder(button)
end


RS.Template.UpdatePlayerSelectorByPlayerData=function(frame,playerdata)
	local playerlist=frame.PlayerList
	for i=1,table.getn(playerlist) do
		local button=playerlist[i]
		if(button.PlayerData and button.PlayerData.Name==playerdata.Name)then
			RS_Template_SetPlayerFrame(button,playerdata)
			return
		end
	end
end
RS.Template.RemovePlayerSelectorByPlayerData=function(frame,playerdata)
	local playerlist=frame.PlayerList
	for i=1,table.getn(playerlist) do
		local button=playerlist[i]
		if(button.PlayerData and button.PlayerData.Name==playerdata.Name)then
			RS_Template_ClearPlayerFrame(button)
			return
		end
	end
end

RS.Template.SetPlayerSelector=function(frame,playerdatalist)
	local playerlist=frame.PlayerList
	local selectedplayer=frame.SelectedPlayer
	local max=table.getn(playerdatalist)
	if(max>frame.PlayerCount)then
		max=frame.PlayerCount
	end

	for i=1,max do
		RS_Template_SetPlayerFrame(playerlist[i],playerdatalist[i],selectedplayer[playerdatalist[i].Name])
	end
	for i=max+1,table.getn(playerlist) do
		RS_Template_ClearPlayerFrame(playerlist[i])
	end
end

RS.Template.CreatePlayerSelector=function(parent,width,height,buttonwidth,buttonheight)
	local frame=RS.Template.CreateFrame(parent,width,height)

	frame.PlayerList={}
	
	--create button
	local CreatePlayerButton=function()
		local dwidth=buttonwidth-5
		local dheight=buttonheight-5
		local button=RS.Template.CreateButton(frame,dwidth,dheight)
		button.Label:SetText("+")
		button.Label:ClearAllPoints()
		button.Label:SetFont(RS.TextFont,10,RS.TextMode)
		button.Label:SetPoint("TOPLEFT",button,"TOPLEFT",0,0)
		button.Label:SetPoint("TOPRIGHT",button,"TOPRIGHT",0,0)

		local halfheight=dheight/2

		button.PortraitTexture=button:CreateTexture(nil,"ARTWORK")
		button.PortraitTexture:SetPoint("TOPLEFT",button,"TOPLEFT")
		button.PortraitTexture:SetPoint("BOTTOMRIGHT",button,"BOTTOMRIGHT")
		button.PortraitTexture:SetHeight(halfheight)

		button.RoleTexture=button:CreateTexture(nil,"OVERLAY")
		button.RoleTexture:SetPoint("BOTTOMLEFT",button,"BOTTOMLEFT")
		button.RoleTexture:SetPoint("BOTTOMRIGHT",button,"BOTTOM")
		button.RoleTexture:SetHeight(halfheight)
		button.RoleTexture:SetTexture(RS.RoleTexture)
		button.RoleTexture:SetTexCoord(0,0,0,0)

		button.SpecTexture=button:CreateTexture(nil,"OVERLAY")
		button.SpecTexture:SetPoint("BOTTOMLEFT",button,"BOTTOM")
		button.SpecTexture:SetPoint("BOTTOMRIGHT",button,"BOTTOMRIGHT")
		button.SpecTexture:SetHeight(halfheight)

		RS.Template.CreateLinerBorder(button,2)
		RS.Template.HideLinerBorder(button)

		button:HookScript("OnEnter",function(self)
			if(self.PlayerData)then
				GameTooltip:SetOwner(self, "ANCHOR_TOP")
				GameTooltip:AddLine(self.PlayerData.Name)
				for spellid,cooldown in pairs(self.PlayerData.SpellList) do
					local name, _,icon=GetSpellInfo(spellid)
					if(name)then
						GameTooltip:AddLine(name.."("..cooldown.."s)")
						GameTooltip:AddTexture(icon)
					end
				end
				GameTooltip:Show()
			end
		end)
		button:HookScript("OnLeave",function(self)
			if(GameTooltip:IsOwned(self))then
				GameTooltip:Hide()
			end
		end)

		button.PlayerData=nil

		button:HookScript("OnMouseUp",function(self,mousebutton)
			if(button.PlayerData==nil)then
				RS.SetPage("CustomPlayer",parent.Scroll.Dragger,function(name,class)
					local playerdata=RS.SpecAndSpells.GeneratePlayerData(name,class)
					RS_Template_SetPlayerFrame(self,playerdata)
					RS.RunFunc(frame.OnPlayerAddedFunc,playerdata)
				end)
			elseif(mousebutton=="LeftButton")then
				if(frame.SelectedPlayer[self.PlayerData.Name])then
					frame.SelectedPlayer[self.PlayerData.Name]=nil
					RS.Template.HideLinerBorder(self)
					RS.RunFunc(frame.OnPlayerUncheckFunc,self.PlayerData)
				else
					frame.SelectedPlayer[self.PlayerData.Name]=self.PlayerData
					RS.Template.ShowLinerBorder(self)
					RS.RunFunc(frame.OnPlayerCheckFunc,self.PlayerData)
				end
			elseif(mousebutton=="RightButton")then
				if(frame.SelectedPlayer[self.PlayerData.Name])then
					frame.SelectedPlayer[self.PlayerData.Name]=nil
					RS.RunFunc(frame.OnPlayerUncheckFunc,self.PlayerData)
				end
				RS_Template_ClearPlayerFrame(self)
				RS.RunFunc(frame.OnPlayerRemovedFunc,self.PlayerData)
			end
		end)

		return button
	end

	local perrow=floor(width/buttonwidth)
	local row=floor(height/buttonheight)
	local count=0
	local framelist={}
	for i=1,row do
		for j=1,perrow do
			count=count+1
			framelist[count]=CreatePlayerButton()
			framelist[count]:SetPoint("TOPLEFT",frame,"TOPLEFT",buttonwidth*(j-1)+5,buttonheight*(1-i)-5)
		end
	end

	frame.PlayerList=framelist
	frame.PlayerCount=count
	frame.SelectedPlayer={}

	frame:HookScript("OnMouseDown",function(...)
		parent:GetScript("OnMouseDown")(...)
	end)
	frame:HookScript("OnMouseUp",function(...)
		parent:GetScript("OnMouseUp")(...)
	end)

	return frame
end
